function par = assignDesignParameters()
%ASSIGNDESIGNPARAMETERS Assign design parameters for the computation of quadrotor maneuvers.
%   par = assignDesignParameters() assigns all design parameters that are
%   needed for the computation of quadrotor maneuvers satisfying
%   Pontryagin's minimum principle using the function computeManeuver().

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Initial control inputs indicators
par.uR0_ind = 1;	% 0 => uR(0) = -1; 1 => uR(0) = +1
par.uT0_ind = 0;	% 0 => uT(0) = uT_min; 1 => uT(0) = uT_max

% Singular arcs switches
par.uR_SAswitch = [1,1];    % eg. [0,1,1] => uR changes sign after singular
                            %                arc of second and third switch

% Integrator settings
par.ODE_solver = 'ode113';          % ODE solver
par.ODE_options = odeset(...        % ODE solver options
    'RelTol',1e-7,...
    'AbsTol',1e-10);
par.ODE_plot_meshsize = 10000;      % mesh size for plotting (approximately)
par.ODE_residual_meshsize = 1000;   % mesh size when calculation the residual
                                    % => important for accuracy of the linear
                                    % interpolations of x when computing the
                                    % condition matrix equation

% Switching time optimization parameters
par.STO_nR = 2;                 % number of uR switches
par.STO_nT = 4;                 % number of uT switches
par.STO_T0 = 3;                 % initial maneuver duration
par.STO_W = diag([10,10,1,1,1]);% weights of the states for state residual
par.STO_Wc = 1000;              % weight of the conditions for condition residual
par.STO_deltaT = 0.1;           % increasing value of T if residual not yet convex
par.STO_TolConv = 3;            % tolerance below which residual is convex
par.STO_TolRes = 1e-6;          % tolerance for residual = 0
par.STO_sym = 0;                % 1 => symmetric maneuver assumed
par.STO_deltaT_ini = -0.1;      % value added to the inital maneuver duration T_ini
                                % (can be used to reduce interations during
                                % STO or (if negative) to make sure that T_ini
                                % is too small to complete the maneuver)
par.STO_allowLessSwitches = 0;  % allow that switches move out of the maneuver
                                % interval during optimization
par.STO_options = optimset(...  % optimization options
    'TolX',1e-6,...
    'TolFun',1e-6,...
    'MaxFunEvals',10000,...
    'MaxIter',4000);

% Parameter extraction parameters
par.PE_TolCond = 1e-4;              % tolerance for norm(A*c-r) = 0 (only used for bang-bang)
par.PE_TolDs = 1e-6;                % tolerance for singular arc (Ds = 0)

% BVP solver parameters
par.BVP_W = diag([10,10,1,1,1]);    % weights of the states for residual
par.BVP_Wc = 1000;                  % weight of the conditions for condition residual
par.BVP_TolRes = 1e-6;              % tolerance for residual = 0
par.BVP_allowLessSwitches = 0;      % allow that switches move out of the maneuver
                                    % interval during optimization (if
                                    % bang-singular maneuver neglected for uT)
par.BVP_options = optimset(...      % optimization options
    'TolX',1e-6,...
    'TolFun',1e-6,...
    'MaxFunEvals',15000,...
    'MaxIter',5000);

% Plotting options (put -1 for no plotting and better performance)
par.stateFig = -1;   % figure of state trajectories (always the trajectories with
                     % the smallest residual within the current minimization are plotted)
par.inputFig = -1;   % figure of input trajectories (always the trajectories with
                     % the smallest residual within the current minimization are plotted)
par.residualFig = 3; % figure of the plot of the minimized residual for each
                     % time-step of the switching time optimization

end
