function par = assignInitialParameters(par,par_ini)
%ASSIGNINITIALPARAMETERS Assign initial parameters for the computation of quadrotor maneuvers.
%   par = assignInitialParameters(par,par_ini) assigns initial parameters that are
%   stored in par_ini to par. The parameters are needed for the computation
%   of quadrotor maneuvers satisfying Pontryagin's minimum principle using
%   the function computeManeuver().

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Initial maneuver duration
if ~isempty(par_ini.T)
    par.STO_T0 = par_ini.T;
    par.T_ini = par_ini.T;
else
    par.T_ini = [];
end

% Initial constants
if ~isempty(par_ini.c)
    par.c_ini = par_ini.c;
else
    par.c_ini = [];
end

% Initial uR definitions
validWithZeroSwitches = 0; % check if initial maneuver has zero switches in uR
if ~isempty(par_ini.nR) && ~isempty(par_ini.uR0_ind) && ...
   isempty(par_ini.T_uR) && isempty(par_ini.Ds_uR) && ...
   isempty(par_ini.uR_SAswitch)
    if par_ini.nR == 0, validWithZeroSwitches = 1; end
end
if validWithZeroSwitches || ...
   (~isempty(par_ini.Ds_uR) && ~isempty(par_ini.T_uR) && ...
   ~isempty(par_ini.uR0_ind) && ~isempty(par_ini.uR_SAswitch))
    par.T_uR_ini = par_ini.T_uR;                % initial switching times of uR
    par.Ds_uR_ini = par_ini.Ds_uR;              % initial singular arc durations
    par.uR_SAswitch_ini = par_ini.uR_SAswitch;  % initial uR singular arcs switches
    par.uR0_ind_ini = par_ini.uR0_ind;          % initial uR initial indicator
    par.STO_nR = length(par.T_uR_ini);          % initial number of switches in uR
else
    par.T_uR_ini = [];
    par.Ds_uR_ini = [];
    par.uR_SAswitch_ini = [];
    par.uR0_ind_ini = [];
end

% Initial uT definitions
validWithZeroSwitches = 0; % check if initial maneuver has zero switches in uT
if ~isempty(par_ini.nT) && ~isempty(par_ini.uT0_ind) && ...
   isempty(par_ini.T_uT)
    if par_ini.nT == 0, validWithZeroSwitches = 1; end
end
if validWithZeroSwitches || ...
   (~isempty(par_ini.T_uT) && ~isempty(par_ini.uT0_ind))
    par.T_uT_ini = par_ini.T_uT;        % initial switching times of uT
    par.uT0_ind_ini = par_ini.uT0_ind;  % initial uT initial indicator
    par.STO_nT = length(par.T_uT_ini);  % initial number of switches in uT
else
    par.T_uT_ini = [];
    par.uT0_ind_ini = [];
end

% Adjust T_ini and scale switching times accordingly
if ~isempty(par.T_ini)
    if ~isempty(par.T_uR_ini)
        par.T_uR_ini = par.T_uR_ini*(par.T_ini+par.STO_deltaT_ini)/par.T_ini;
    end
    if ~isempty(par.Ds_uR_ini)
        par.Ds_uR_ini = par.Ds_uR_ini*(par.T_ini+par.STO_deltaT_ini)/par.T_ini;
    end
    if ~isempty(par.T_uT_ini)
        par.T_uT_ini = par.T_uT_ini*(par.T_ini+par.STO_deltaT_ini)/par.T_ini;
    end
    par.T_ini = par.T_ini+par.STO_deltaT_ini;
    par.STO_T0 = par.T_ini;
end

end
