function par = computeManeuver(x0,xT,uT_min,uT_max,par_ini,isBangbang)
%COMPUTEMANEUVER Compute quadrotor maneuvers satisfying Pontryagin's minimum principle.
%   par = computeManeuver(x0,xT,u_min,u_max,par_ini,isBangbang) computes a quadrotor
%   maneuver with initial state x0 and final state xT, that satisfies Pontryagin's
%   minimum principle with respect to time-optimality. uT_min and uT_max 
%   represent the dimensionless thrust input boundaries. The state and input
%   trajectories are saved in the struct par. Using the struct par_ini,
%   several initial values for the optimization can be provided.

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Display
disp(['Compute time-optimal maneuver from ',...
      'x0 = [',num2str(x0(1),3),',',num2str(x0(2),3),',',...
               num2str(x0(3),3),',',num2str(x0(4),3),',',...
               num2str(x0(5),3),'] to ',...
      'xT = [',num2str(xT(1),3),',',num2str(xT(2),3),',',...
               num2str(xT(3),3),',',num2str(xT(4),3),',',...
               num2str(xT(5),3),']']);
disp(['with uT_min = ',num2str(uT_min,3),...
      ' and uT_max = ',num2str(uT_max,3),':']);

% Load design parameters
par = assignDesignParameters();

% Assign initial parameters (! some design parameters might be overwritten !)
if ~isempty(par_ini)
    par = assignInitialParameters(par,par_ini);
    par.hasIC = 1;
else
    par.hasIC = 0;
end

% Assign quadrotor parameters
par.uR_min = -1;        % minimum dimensionless rotational velocity
par.uR_max = +1;        % maximum dimensionless rotational velocity
par.uT_min = uT_min;    % minimum dimensionless thrust
par.uT_max = uT_max;    % maximum dimensionless thrust

% Assign initial and final states
par.x0 = x0;    % dimensionless initial quadrotor state
par.xT = xT;	% dimensionless final quadrotor state

% Assign if maneuver is assumed to be bang-bang
par.isBangbang = isBangbang;

% Set cost per time that is minimized
par.cost = 0.1; % set to 0.1 because then g(x,u) = 1 [1/s] for theta_max = 10 [rad/s]

% Flag par as non-dimensionalized
par.hasDim = 0;

% Initialize trajectories and constants
par.t = []; par.x = [];
par.u = []; par.Phi = [];
par.c = [];

% Switching time optimization
disp(['  ->start switching time optimization with T = ',num2str(par.STO_T0,6)]);
par = executeSTO(par);
if par.STO_Res > par.STO_TolRes
    disp('  ->The switching time optimization could not find a maneuver');
    disp('    that satisfies the specified tolerance. Be carefull with');
    disp('    the result.');
end

% save('par_after_STO.mat','par');
% load ('par_after_STO.mat');

% Parameter extraction
if par.isBangbang
    disp('  ->start parameter extraction');
    par = exctractParameters(par,0,0);
    if isempty(par.c)
        disp('  ->No constant vector c that satisfied the condition matrix');
        disp('    equation A*c = r with the specified tolerance could be found.');
        disp('    Adjust the design parameters and restart the computation.');
        disp('Computation aborted.');
        par = [];
        return;
    end
end

% save('par_after_PE.mat','par');
% load ('par_after_PE.mat');

% BVP solver
disp('  ->start BVP solver');
if par.isBangbang
    par = solveBVP(par,1,1);
else
    par = solveBVP(par,0,1);
end
if par.BVP_Res > par.BVP_TolRes
    disp('  ->The BVP solver could not find a maneuver that satisfies');
    disp('    the specified tolerance. Be carefull with the result.');
end

% save('par.mat','par');

% Display
disp('Terminated successfully.');

end

