function varargout = exctractParameters(par,uR_pont,uT_pont)
%EXCTRACTPARAMETERS Extract the constants c of a quadrotor maneuver.
%   par = exctractParameters(par,uR_pont,uT_pont) extracts the parameters basd on
%   the quadrotor maneuver saved in the struct par. The constants are stored in
%   par.c of the returned struct.
%   [A,r] = exctractParameters(par,uR_pont,uT_pont) returns the matrix A and the
%   vector r of the condition matrix equation A*c = r.

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Check if valid inputs
if isempty(par.c) && (uR_pont || uT_pont)
    error('uR and uT cannot be computed since c is unknown');
end

% Compute discontinuities in uR
T_uR_dis = [par.T_uR,...
            par.T_uR(par.Ds_uR > par.PE_TolDs)+...
            par.Ds_uR(par.Ds_uR > par.PE_TolDs)];
T_uR_dis = unique(T_uR_dis);
T_uR_dis = T_uR_dis(T_uR_dis > 0);
T_uR_dis = T_uR_dis(T_uR_dis < par.T);
par.nRs = length(T_uR_dis) - par.nR; % overwrite number of singular arcs to
                                     % avoid problems with different tolerances

% Compute Hamiltonian conditions on PhiR
if ~uR_pont
    A1 = zeros(par.nR+par.nRs,4); r1 = zeros(par.nR+par.nRs,1);
    for i = 1:(par.nR+par.nRs)
        b = b_H(T_uR_dis(i),par,-1,uR_pont,uT_pont);  % compute linear coefficients b
        A1(i,:) = b(2:5); r1(i) = -b(1);              % augment condition matrix equation
    end
else
    A1 = []; r1 = [];
end

% Compute integral conditions on PhiR
if ~uR_pont
    A2 = zeros(par.nR+par.nRs+1,4); r2 = zeros(par.nR+par.nRs+1,1);
    if (par.nR+par.nRs) == 0
        d = d_I(0,par.T,par,uT_pont);           % compute linear coefficients d
        b0 = b_H(0,par,-1,uR_pont,uT_pont);     % compute linear coefficients b (take uR from
        bT = b_H(par.T,par,-1,uR_pont,uT_pont); % regular iterval to avoid division by zero)
        A2(1,:) = d + b0(2:5) - bT(2:5); r2(1) = bT(1) - b0(1);
    else
        d = d_I(0,T_uR_dis(1),par,uT_pont); % compute linear coefficients d
        b0 = b_H(0,par,-1,uR_pont,uT_pont); % compute linear coefficients b (avoid singular arcs)
        A2(1,:) = d + b0(2:5); r2(1) = -b0(1);
        for i = 1:(par.nR+par.nRs-1)
            d = d_I(T_uR_dis(i),T_uR_dis(i+1),par,uT_pont); % compute linear coefficients d
            A2(i+1,:) = d;
        end
        d = d_I(T_uR_dis(end),par.T,par,uT_pont); % compute linear coefficients d
        bT = b_H(par.T,par,-1,uR_pont,uT_pont);   % compute linear coefficients b (avoid singular arcs)
        A2(end,:) = d - bT(2:5); r2(end) = bT(1);
    end
else
    A2 = []; r2 = [];
end

% Conditions on derivative of PhiR
if ~uR_pont
    A3 = zeros(2*par.nRs,4); r3 = zeros(2*par.nRs,1);
    j = -1;
    for i = 1:par.nR
        if par.Ds_uR(i) > par.PE_TolDs
            j = j + 1;
            h = h_P(par.T_uR(i),par,uT_pont); % compute linear coefficients h
            A3(2*j+1,:) = h;
            h = h_P(par.T_uR(i)+par.Ds_uR(i),par,uT_pont); % compute linear coefficients h
            A3(2*j+2,:) = h;
        end
    end
else
    A3 = []; r3 = [];
end

% Conditions on PhiT
if ~uT_pont
    A4 = zeros(par.nT,4); r4 = zeros(par.nT,1);
    for i = 1:par.nT
        k = k_P(par.T_uT(i),par); % compute linear coefficients k
        A4(i,:) = k;
    end
else
    A4 = []; r4 = [];
end

% Aggregate condition equations to obtain condition matrix equation
A = [A1;A2;A3;A4]; r = [r1;r2;r3;r4];
for i = 1:size(A,1)
    n = sqrt(sum(A(i,:).^2) + r(i)^2); % normalize rows of the matrix condition
    if n > 0                           % equation to give approximately the same
        A(i,:) = A(i,:)/n;             % weight to all linear conditions
        r(i) = r(i)/n;
    end
end

% Return condition matrix equation if two output arguments
if nargout == 2
    varargout(1) = {A};
    varargout(2) = {r};
    return;
end

% Compute constant vector c
if isempty(A) || isempty(r)
    error('could not compute the constants c since no conditions found');
else
    par.c = pinv(A)*r;
    if norm(A*par.c-r) > par.PE_TolCond, par.c = []; end
end

% Return par with values for c, A, and r
par.A = A; par.r = r;
varargout(1) = {par};

end

%% ------------------------------------------------------------------------

function b_out = b_H(t,par,sing,uR_pont,uT_pont)
%B_H Return the linear coefficients b.

% Interpolate current state vector x
x = interp1f(par.t,par.x,t);

% Compute current control inputs
uR = rotInput(t,x,par,sing,uR_pont);
uT = thrustInput(t,x,par,uT_pont);

% Return coefficients
b_out = [   -par.cost
            -x(2)+t*uT*sin(x(5))
            -uT*sin(x(5))
            -x(4)+t*(uT*cos(x(5))-1)
            -uT*cos(x(5))+1];
b_out = b_out/uR;

end

%% ------------------------------------------------------------------------

function d_out = d_I(t1,t2,par,uT_pont)
%D_I Return the linear coefficients d.

% Define sorted integration segments
if uT_pont
    T_dis = unique([t1,t2]);
else
    T_uT = par.T_uT(par.T_uT > t1);
    T_uT = T_uT(T_uT < t2);
    T_dis = unique([t1,t2,T_uT]);
end

% Add event function to integration options if required
if uT_pont
    ODE_options = odeset(par.ODE_options,'Events',@eventFunctionPhiT);
else
    ODE_options = par.ODE_options;
end

% Define ODE solver function
solver = str2func(par.ODE_solver);

% Compute coefficients by piecewise integration of the smooth segments
t = t1; d = zeros(1,4);
for i = 1:length(T_dis)-1

    % Avoid crash of ODE solver if segment is too short
    if abs(T_dis(i+1)-T_dis(i)) > 1e-10
        
        % Integrate over current segment (loop necessary since restart of
        % integration required if it was aborted due to discontinuity)
        while 1
    
            % Define initial inputs of current segment
            par.u_const = [0,thrustInput(t,[],par,uT_pont)]; % u_const(1) not used
    
            % Integrate quadrotor dynamics over current segment (and abort if
            % discontinuity occurs)
            [t_add,d_add] = solver(@d_ODE,[t,T_dis(i+1)],d,ODE_options,par);
            
            % Read new values
            t = t_add(end); d = d_add(end,:);

            % Check if segment integration completed
            if t >= T_dis(i+1), break; end
            
        end
        
    end
    
end

% Clear inputs
par.u_const = [];

% Return coefficients
d_out = d';

end

%% ------------------------------------------------------------------------

function dddt = d_ODE(t,~,par)
%D_ODE ODE for the condition matrix coefficients d.

% Interpolate current pitch x(5)
x5 = interp1f(par.t,par.x(:,5),t);

% Read current thrust input uT
uT = par.u_const(2);

% Return current derivative
dddt = [t*uT*cos(x5)
        -uT*cos(x5)
        -t*uT*sin(x5)
        uT*sin(x5)];
    
end
        
%% ------------------------------------------------------------------------

function h_out = h_P(t,par,uT_pont)
%H_P Return the linear coefficients h.

% Interpolate current state vector x
x = interp1f(par.t,par.x,t);

% Compute current thrust input uT
uT = thrustInput(t,x,par,uT_pont);

% Return coefficients
h_out = [   t*uT*cos(x(5))
            -uT*cos(x(5))
            -t*uT*sin(x(5))
            uT*sin(x(5))];
        
end

%% ------------------------------------------------------------------------

function k_out = k_P(t,par)
%K_P Return the linear coefficients k.

% Interpolate current pitch x(5)
x5 = interp1f(par.t,par.x(:,5),t);

% Return coefficients
k_out = [   -t*sin(x5)
            sin(x5)
            -t*cos(x5)
            cos(x5)];
        
end

%% ------------------------------------------------------------------------

function [value,isterminal,direction] = eventFunctionPhiT(t,~,par)
%EVENTFUNCTIONPHIT Detect control switches in uT and abort integration

% Interpolate current pitch x(5)
x5 = interp1f(par.t,par.x(:,5),t);

% Detect control switches of uT, i.e. PhiT = 0                                                           %Phi_1 = 0
value = (par.c(2)-par.c(1)*t)*sin(x5)+(par.c(4)-par.c(3)*t)*cos(x5);

% Stop integration when switch occurs
isterminal = 1;

% Zero-crossing direction does not matter
direction = 0;

end