function par = executeSTO(par)
%EXECUTESTO Execute switching time optimization to obtain quadrotor maneuvers.
%   par = executeSTO(par) executes a switching time optimization algorithm
%   to obtain a quadrotor maneuver that is a candidate for time-optimality.
%   The resulting switches and maneuver trajectories are stored in the
%   struct par.

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Initialize global variables with plotting infos
if par.stateFig > -1 || par.inputFig > -1
    global minimumResidual; %#ok<TLEV> % minimum residual of the current minimization
end

% Loop that increases duration T until maneuver can be completed
par.T = par.STO_T0; deltaT_min = par.STO_deltaT;
last_res_min = []; last_deltaT = []; last_par = [];
plot_res_min = []; plot_T = [];
while 1
    
    % Assign initial switching times and singular arc durations
    % (second half ignored if symmetric)
    if validInitialConditions(par)
        [par.T_uR,IX] = sort(par.T_uR_ini*par.T/par.T_ini);
        par.Ds_uR = par.Ds_uR_ini(IX)*par.T/par.T_ini;
        par.uR_SAswitch = par.uR_SAswitch_ini(IX);
        par.uR0_ind = par.uR0_ind_ini;
        par.T_uT = sort(par.T_uT_ini*par.T/par.T_ini);
        par.uT0_ind = par.uT0_ind_ini;
    else
        par.Ds_uR = ones(1,par.STO_nR)*par.T/(8*par.STO_nR); % 1/8 of time in singular arcs
        par.T_uR = linspace(0,par.T,par.STO_nR+2);  % switches distributed equally
        par.T_uR = par.T_uR(2:end-1);
        par.T_uR = par.T_uR-0.5*par.Ds_uR;          % adjust for singular arcs
        par.T_uT = linspace(0,par.T,par.STO_nT+2);  % switches distributed equally
        par.T_uT = par.T_uT(2:end-1);
    end
    
    % Assign initial constants
    if par.hasIC && ~isempty(par.c_ini)
        par.c = par.c_ini;
    else
        par.c = zeros(4,1); % set to zero vector, but computation will
    end                     % probably fail if no initial value for
                            % constant vector c is given and if maneuver
                            % has singular arcs
                            
    % Define function to minimize
    if ~par.STO_sym % optimize all switches if maneuver not symmetric
        if par.isBangbang
            opt_vars = ones(1,par.STO_nR+par.STO_nT);
            res = @(opt_vars) residualFunction(par,[...
                opt_vars(1:par.STO_nR).*par.T_uR,...
                zeros(1,par.STO_nR),... % singular arc duration always zero
                opt_vars(par.STO_nR+1:par.STO_nR+par.STO_nT).*par.T_uT,...
                zeros(1,4)],... % placeholder (constants not used)
             	[],'STO',0,0);
        else
            opt_vars = ones(1,2*par.STO_nR+par.STO_nT+4);
            res = @(opt_vars) residualFunction(par,...
             	opt_vars.*[par.T_uR,par.Ds_uR,par.T_uT,par.c'],...
                [],'STO',0,0);
        end
    else % optimize only first half of switches if maneuver symmetric
        if par.isBangbang
            opt_vars = ones(1,par.STO_nR/2+par.STO_nT/2);
            res = @(opt_vars) residualFunction(par,[...
                opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2),...
                par.T-opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2)-...
                    opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2),...
                zeros(1,par.STO_nR),... % singular arc duration always zero
                opt_vars(par.STO_nR/2+1:par.STO_nR/2+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2),...
                par.T-opt_vars(par.STO_nR/2+1:par.STO_nR/2+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2),...
                zeros(1,4)],... % placeholder (constants not used)
                [],'STO',0,0);
        else
            opt_vars = ones(1,par.STO_nR+par.STO_nT/2+4);
            res = @(opt_vars) residualFunction(par,[...
                opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2),...
                par.T-opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2)-...
                    opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2),...
                opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2),...
                opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2),...
                opt_vars(par.STO_nR+1:par.STO_nR+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2),...
                par.T-opt_vars(par.STO_nR+1:par.STO_nR+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2),...
                opt_vars(par.STO_nR+par.STO_nT/2+1:end).*par.c'],...
                [],'STO',0,0);
        end
    end
    
    % Clear the minimum residual if needed for state or input plotting
    if par.stateFig > -1 || par.inputFig > -1
        minimumResidual = inf;
    end

    % Minimize residual (fminsearch used since gradient methods problematic
    % due to numerical noise caused by integration of quadrotor dynamics)
    [opt_vars,res_min] = fminsearch(res,opt_vars,par.STO_options);
    
    % Update residual plot if necessary
    if par.residualFig > -1
        plot_res_min(end+1) = res_min; %#ok<AGROW>
        plot_T(end+1) = par.T; %#ok<AGROW>
        plotResidual(par,plot_res_min,plot_T,par.residualFig);
        drawnow;
    end
    
    % Read optimal switching times and constants
    if ~par.STO_sym
        if par.isBangbang
            par.T_uR = opt_vars(1:par.STO_nR).*par.T_uR;
            par.Ds_uR = zeros(1,par.STO_nR);
            par.T_uT = opt_vars(par.STO_nR+1:par.STO_nR+par.STO_nT).*par.T_uT;
            par.c = [];
        else
            par.T_uR = opt_vars(1:par.STO_nR).*par.T_uR;
            par.Ds_uR = opt_vars(par.STO_nR+1:2*par.STO_nR).*par.Ds_uR;
            par.T_uT = opt_vars(2*par.STO_nR+1:2*par.STO_nR+par.STO_nT).*par.T_uT;
            par.c = opt_vars(2*par.STO_nR+par.STO_nT+1:end)'.*par.c;
        end
    else
        if par.isBangbang
            par.T_uR = [...
                opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2),...
                par.T-opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2)-...
                    opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2)];
            par.Ds_uR = zeros(1,par.STO_nR);
            par.T_uT = [...
                opt_vars(par.STO_nR/2+1:par.STO_nR/2+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2),...
                par.T-opt_vars(par.STO_nR/2+1:par.STO_nR/2+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2)];
            par.c = [];
        else
            par.T_uR = [...
                opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2),...
                par.T-opt_vars(1:par.STO_nR/2).*par.T_uR(1:par.STO_nR/2)-...
                    opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2)];
            par.Ds_uR = [...
                opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2),...
                opt_vars(par.STO_nR/2+1:par.STO_nR).*par.Ds_uR(1:par.STO_nR/2)];
            par.T_uT = [...
                opt_vars(par.STO_nR+1:par.STO_nR+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2),...
                par.T-opt_vars(par.STO_nR+1:par.STO_nR+par.STO_nT/2).*par.T_uT(1:par.STO_nT/2)];
            par.c = opt_vars(par.STO_nR+par.STO_nT/2+1:end)'.*par.c;
        end
    end
    
    % Check if residual close enough to zero
    if res_min < par.STO_TolRes
        par.STO_Res = res_min;
        break;
    end
    
    % Check if residual has increased and exit STO if so
    if ~isempty(last_res_min) && ~isempty(last_par)
        if res_min > last_res_min
            par = last_par;
            par.STO_Res = res_min;
            break;
        end
    end
    
    % Store last par in case the residual increases during next iteration
    last_par = par;
    
    % Store optimization parameters as initial value of next iteration
    par.T_ini = par.T;
    par.T_uR_ini = par.T_uR;
    par.Ds_uR_ini = par.Ds_uR;
    par.T_uT_ini = par.T_uT; 
    par.c_ini = par.c;
    
    % Increase maneuver duration T
    if (isempty(last_res_min) && isempty(last_deltaT)) || res_min > par.STO_TolConv
        deltaT = par.STO_deltaT; % increase by the constant value as defined
    else
        deltaT = last_deltaT/(last_res_min/res_min-1); % use secant method
        deltaT = min(deltaT,deltaT_min);  % make sure that deltaT does not
                                          % increase due to numerical noise
        if deltaT < deltaT_min, deltaT_min = deltaT; end
    end
    par.T = par.T + deltaT;
    last_res_min = res_min; last_deltaT = deltaT; % store values for next iteration
    disp(['    ->current residual = ',num2str(res_min,4),...
          ' => increase maneuver duration to T = ',num2str(par.T,6)]);

end

% Clear global variables with plotting infos
if par.stateFig > -1 || par.inputFig > -1
    clear global minimumResidual;
end

% Eliminate nonrelevant switches
if mod(sum(par.T_uR <= 0),2) == 1 % adjust uR(0) if necessary
    par.uR0_ind = ~par.uR0_ind;
end
if mod(sum(par.T_uT <= 0),2) == 1 % adjust uT(0) if necessary
    par.uT0_ind = ~par.uT0_ind;
end
par.Ds_uR = par.Ds_uR(par.T_uR > 0); par.Ds_uR = par.Ds_uR(par.T_uR < par.T);
par.T_uR = par.T_uR(par.T_uR > 0); par.T_uR = par.T_uR(par.T_uR < par.T);
par.T_uT = par.T_uT(par.T_uT > 0); par.T_uT = par.T_uT(par.T_uT < par.T);

% Store number of switches
par.nR = length(par.T_uR);              % number of switches in uR
par.nRs = sum(par.Ds_uR>par.PE_TolDs);  % number of singular arcs
par.nT = length(par.T_uT);              % number of switches in uT

% Compute resulting quadrotor trajectories
[par.t,par.x] = integrateQuadrotorDynamics(par,par.ODE_options,0,...
                                           par.ODE_plot_meshsize,0,0);
par.u = zeros(length(par.t),2);
for i = 1:length(par.t)
    par.u(i,1) = rotInput(par.t(i),par.x(i,:),par,0,0);
    par.u(i,2) = thrustInput(par.t(i),par.x(i,:),par,0);
end

end

%% ------------------------------------------------------------------------

function valid = validInitialConditions(par)
%VALIDINITIALCONDITIONS Check if initial conditions are exist.
%   valid = validInitialConditions(par) checks if initial conditions are
%   stored in the struct par and if they are complete and valid.

% Check if initial conditions exist
if ~par.hasIC
    valid = 0;
    return;
end

% Check if initial conditions are valid
if isempty(par.T_ini) || ...
(isempty(par.T_uR_ini) && par.STO_nR > 0) || ...
(isempty(par.Ds_uR_ini) && par.STO_nR > 0) || ...
(isempty(par.uR_SAswitch_ini) && par.STO_nR > 0) || ...
isempty(par.uR0_ind_ini) || ...
(isempty(par.T_uT_ini) && par.STO_nT > 0) || ...
isempty(par.uT0_ind_ini)
    valid = 0;
    return;
end

% Check if number of switches is correct
if length(par.T_uR_ini) == par.STO_nR && ...
length(par.Ds_uR_ini) == par.STO_nR && ...
length(par.uR_SAswitch_ini) == par.STO_nR && ...
length(par.T_uT_ini) == par.STO_nT
    valid = 1;
else
    valid = 0;
end

end


