function [t,x] = integrateQuadrotorDynamics(par,ODE_options,reverse,meshsize,uR_pont,uT_pont)
%INTEGRATEQUADROTORDYNAMICS Compute the quadrotor trajectory by ODE integration.
%   [t,x] = integrateQuadrotorDynamics(par,ODE_options,reverse,meshsize,uR_pont,uT_pont)
%   integrates the quadrotor dynamics from 0 to par.T, applying the control inputs
%   defined by par.T_uR, par.Ds_uR, and par.T_uT. The constats par.c must be stored
%   for the computation of the singular input. The maneuver trajectory is
%   returned in t and x. The boolean variabel reverse indicates if the
%   system is integrated forward in time or backwards. If the meshsize is
%   empty, then the variable ODE-solver meshsize is returned.

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Get and sort all unique discontinuities
if uR_pont && uT_pont
    T_dis = unique([0,par.T]);
elseif uR_pont && ~uT_pont
    T_dis = unique([0,par.T,par.T_uT]);
elseif ~uR_pont && uT_pont
    T_dis = unique([0,par.T,par.T_uR,par.T_uR+par.Ds_uR]);
else
    T_dis = unique([0,par.T,par.T_uR,par.T_uR+par.Ds_uR,par.T_uT]);
end

% Reverse discontinuities if reverse integration is required
if reverse, T_dis = T_dis(end:-1:1); end

% Eliminate nonrelevant discontinuities
T_dis = T_dis(T_dis >= 0); T_dis = T_dis(T_dis <= par.T);

% Assign initial time and initial quadrotor state
if ~reverse
    t_start = 0;
    t_eps = 0; % used to make sure that u is computed for the following segment
    x_start = par.x0;
else
    t_start = par.T;
    t_eps = -10*eps; % used to make sure that u is computed for the following segment
    x_start = par.xT;
end

% Compute initial value of the switching function PhiR if c known
if ~isempty(par.c)
    t_ref = t_start;
    if ~any((t_ref >= par.T_uR).*(t_ref < (par.T_uR+par.Ds_uR))) % if not in singular arc
        uR_start = rotInput(t_start,[],par,-1,0); % compute uR based on uR0_ind
        uT_start = thrustInput(t_start,x_start,par,uT_pont);
        b_start = [ -par.cost,...
                    -x_start(2)+t_start*uT_start*sin(x_start(5)),...
                    -uT_start*sin(x_start(5)),...
                    -x_start(4)+t_start*(uT_start*cos(x_start(5))-1),...
                    -uT_start*cos(x_start(5))+1]/uR_start;
        PhiR_start = b_start*[1,par.c']';
    else
        PhiR_start = 0;
    end
end

% Compute initial augmented state vector if c known
if ~isempty(par.c)
    x_start = [x_start;PhiR_start];
end

% Add Jacobian to integration options (only helpful if ODE solver uses Jacobian)
ODE_options = odeset(ODE_options,'Jacobian',@quadrotorJacobian);

% Add event function to integration options if required
if uR_pont && uT_pont
    ODE_options = odeset(ODE_options,'Events',@eventFunction);
elseif uR_pont && ~uT_pont
    ODE_options = odeset(ODE_options,'Events',@eventFunctionPhiR);
elseif ~uR_pont && uT_pont
    ODE_options = odeset(ODE_options,'Events',@eventFunctionPhiT);
end

% Define ODE solver function
solver = str2func(par.ODE_solver);

% Compute trajectory by integrating each smooth segment between the discontinuities
t = t_start; x = x_start';
for i = 1:(length(T_dis)-1)
    
    % Avoid crash of ODE solver if segment is too short
    if abs(T_dis(i+1)-T_dis(i)) > 1e-10
        
        % Integrate over current segment (loop necessary since restart of
        % integration required if it was aborted due to discontinuity)
        while 1

            % Compute initial control inputs of current segment
            t_ref = t_start + t_eps; % make sure that u is computed for the following segment
            if any((t_ref >= par.T_uR).*(t_ref < (par.T_uR+par.Ds_uR))) % if in singular arc
                par.u_const = [0,... % uR=0 means uR=uRsing
                               thrustInput(t_ref,x_start,par,uT_pont)];
            else
                par.u_const = [rotInput(t_ref,x_start,par,-1,uR_pont),...
                               thrustInput(t_ref,x_start,par,uT_pont)];
            end
            
            % Define integrator time-span
            if isempty(meshsize)
                t_span = [t_start,T_dis(i+1)];
            else
                t_span = linspace(t_start,T_dis(i+1),...
                                  max(2,meshsize*abs(T_dis(i+1)-t_start)/par.T));
            end

            % Integrate quadrotor dynamics over current segment (and abort if
            % discontinuity occurs)
            [t_add,x_add] = solver(@quadrotorODE,t_span,x_start,ODE_options,par);

            % Add segment trajectory to maneuver trajectory
            t = [t;t_add(2:end)]; x = [x;x_add(2:end,:)]; %#ok<AGROW>

            % Compute initial conditions of next segment
            t_start = t(end);
            x_start = x(end,:);

            % Check if segment integration completed
            if (t_start >= T_dis(i+1) && ~reverse) || ...
               (t_start <= T_dis(i+1) && reverse)
                break;
            end
            
        end
    
    end
    
end

% Clear inputs
par.u_const = [];

% Reverse maneuver if bachwards integration
if reverse
    t = t(end:-1:1);
    x = x(end:-1:1,:);
end

end

%% ------------------------------------------------------------------------

function dxdt = quadrotorODE(t,x,par)
%QUADROTORODE ODE of the augmented quadrotor model.

% Read current control inputs
if isempty(par.u_const)
    error('the controls must be precomputed before integrating segment');
else
    uR = par.u_const(1);
    uT = par.u_const(2);
end

% Get singular uR if necessary
if uR == 0 
    if ~isempty(par.c)
        uR = rotInput(t,x,par,1,0);
    else
        error('connot compute singular uR since c is missing');
    end
end

% Return current derivative of the quadrotor model
if ~isempty(par.c)
    c = par.c;
    dxdt = [x(2)
            uT*sin(x(5))
            x(4)
            uT*cos(x(5)) - 1
            uR
            (c(1)*t-c(2))*uT*cos(x(5)) + (c(4)-c(3)*t)*uT*sin(x(5))];
else
    dxdt = [x(2)
            uT*sin(x(5))
            x(4)
            uT*cos(x(5)) - 1
            uR];
end
    
end

%% ------------------------------------------------------------------------

function J = quadrotorJacobian(t,x,par)
%OUADROTORJACOBIAN Compute analytical Jacobian of the augmented system

% Compute current thrust input uT
if isempty(par.u_const)
    error('the controls must be precomputed before integrating segment');
else
    uT = par.u_const(2);
end

% Compute Jacobian
if ~isempty(par.c)
    c = par.c;
    J = [   0, 1, 0, 0, 0, 0;
            0, 0, 0, 0, uT*cos(x(5)), 0;
            0, 0, 0, 1, 0, 0;
            0, 0, 0, 0, -uT*sin(x(5)), 0;
            0, 0, 0, 0, 0, 0;
            0, 0, 0, 0, -(c(1)*t-c(2))*uT*sin(x(5)) + (c(4)-c(3)*t)*uT*cos(x(5)),0];
else
    J = [   0, 1, 0, 0, 0;
            0, 0, 0, 0, uT*cos(x(5));
            0, 0, 0, 1, 0;
            0, 0, 0, 0, -uT*sin(x(5));
            0, 0, 0, 0, 0]; 
end
    
end

%% ------------------------------------------------------------------------

function [value,isterminal,direction] = eventFunctionPhiR(~,x,~)
%EVENTFUNCTIONPHIR Detect control switches in uR and abort integration

% Detect control switches of uR, i.e. PhiR = 0                                                           %Phi_1 = 0
value = x(6);

% Stop integration when switch occurs
isterminal = 1;

% Zero-crossing direction does not matter
direction = 0;

end

%% ------------------------------------------------------------------------

function [value,isterminal,direction] = eventFunctionPhiT(t,x,par)
%EVENTFUNCTIONPHIT Detect control switches in uT and abort integration

% Detect control switches of uT, i.e. PhiT = 0                                                           %Phi_1 = 0
value = (par.c(2)-par.c(1)*t)*sin(x(5))+(par.c(4)-par.c(3)*t)*cos(x(5));

% Stop integration when switch occurs
isterminal = 1;

% Zero-crossing direction does not matter
direction = 0;

end

%% ------------------------------------------------------------------------

function [value,isterminal,direction] = eventFunction(t,x,par)
%EVENTFUNCTIONP Detect control switches and abort integration

% Detect control switches of uR, i.e. PhiR = 0                                                           %Phi_1 = 0
value(1) = x(6);

% Detect control switches of uT, i.e. PhiT = 0                                                     %Phi_1 = 0
value(2) = (par.c(2)-par.c(1)*t)*sin(x(5))+(par.c(4)-par.c(3)*t)*cos(x(5));

% Stop integration when switch occurs
isterminal = [1,1];

% Zero-crossing direction does not matter
direction = [0,0];

end

