function plotManeuver(par,fig,varargin)
%PLOTMANEUVER Plot quadrotor maneuver trajectories.
%   plotManeuver(par,fig) plots the maneuver of the quadrotor that is saved
%   in the struct par. Two plots are created: the system states and the
%   inputs (including the switching functions).

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Read input arguments
if nargin == 4
    nR_pont = varargin{1};
    nT_pont = varargin{2};
else
    nR_pont = [];
    nT_pont = [];
end

% Plot state trajectories x(t)
if fig(1) > -1
    figure(fig(1));
    plot(par.t,par.x(:,1:5));
    title('States $x(t)$','Interpreter','latex');
    xlabel('$t$','Interpreter','latex');
    legend({'$x$','$\dot{x}$','$z$','$\dot{z}$','$\theta$'},...
           'Location','NorthWest','Interpreter','latex');
    x_min = min(min(par.x(:,1:5)));
    x_max = max(max(par.x(:,1:5)));
    axis([0,par.T,x_min-0.1*(x_max-x_min),x_max+0.1*(x_max-x_min)]);
end

% Check if control trajectory u(t) has to be plotted
if fig(2) > -1

    % Compute control trajectories u(t) if not yet computed
    if size(par.u,1) ~= length(par.t)
        u = zeros(length(par.t),2);
        for i = 1:length(par.t)
            u(i,1) = rotInput(par.t(i),par.x(i,:),par,0,nR_pont);
            u(i,2) = thrustInput(par.t(i),par.x(i,:),par,nT_pont);
        end
    else
        u = par.u;
    end

    % Compute switching function trajectories Phi(t) if not yet computed
    if size(par.Phi,1) ~= length(par.t)
        if size(par.x,2) == 6 && ~isempty(par.c)
            Phi = [par.x(:,6),...
                       (par.c(2)-par.c(1)*par.t).*sin(par.x(:,5))+...
                       (par.c(4)-par.c(3)*par.t).*cos(par.x(:,5))];
        else
            Phi = [];
        end
    else
        Phi = par.Phi;
    end

    % Plot u(t) and Phi(t)
    figure(fig(2));
    if ~isempty(Phi)
        Phi(:,1) = Phi(:,1)/max(abs(Phi(:,1)))*par.uR_max; % normalize for plotting
        Phi(:,2) = Phi(:,2)/max(abs(Phi(:,2)))*par.uR_max; % normalize for plotting
        plot(par.t,u,par.t,Phi);
        if par.hasDim
            legend({'$\omega$','$F_T/m$','$\Phi_R$','$\Phi_T$'},...
               'Location','NorthWest','Interpreter','latex');
        else
            legend({'$u_R$','$u_T$','$\Phi_R$','$\Phi_T$'},...
               'Location','NorthWest','Interpreter','latex');
        end
    else
        plot(par.t,u);
        if par.hasDim
            legend({'$\omega$','$F_T/m$'},...
                   'Location','NorthWest','Interpreter','latex');
        else
            legend({'$u_R$','$u_T$'},...
                   'Location','NorthWest','Interpreter','latex');
        end
    end
    title('Control Inputs $u(t)$','Interpreter','latex');
    xlabel('$t$','Interpreter','latex');
    u_min = min(min(u));
    u_max = max(max(u));
    axis([0,par.T,u_min-0.1*(u_max-u_min),u_max+0.1*(u_max-u_min)]);
    
end

end

