function res = residualFunction(par,opt_vars,T,step,uR_pont,uT_pont)
%RESIDUALFUNCTION Compute sum of state and condition residual.
%   res = residualFunction(par,opt_vars,T,step,uR_pont,uT_pont) computes the sum of
%   the state residual and the condition residual for the parameters specified
%   in the struct par and in the vector opt_vars. The vector opt_vars must be
%   structured as follows:
%       opt_vars = [T_uR,Ds_uR,T_uT,c]
%   If the inputs opt_vars or T are empty, then the values are taken from
%   the struct par. The variable step indicates the current step of the
%   maneuver computation and must equal one of the following values:
%       step = 'STO' -> switching time optimization
%       step = 'BVP' -> BVP solver

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Assign variables according to current step
switch step
    case 'STO'
        par.nR = par.STO_nR; % needed for call of extractParameters()
        par.nT = par.STO_nT; % needed for call of extractParameters()
        W = par.STO_W;
        Wc = par.STO_Wc;
        allowLessSwitches = par.STO_allowLessSwitches;
    case 'BVP'
        W = par.BVP_W;
        Wc = par.BVP_Wc;
        allowLessSwitches = par.BVP_allowLessSwitches;
    otherwise
        error('invalid input for the variable step in the function residualFunction()');
end

% Read optimizing variables
if uR_pont && uT_pont
    par.T_uR = [];
    par.Ds_uR = [];
    par.T_uT = [];
    par.c = opt_vars';
elseif uR_pont && ~uT_pont
    par.T_uR = [];
    par.Ds_uR = [];
    par.T_uT = opt_vars(1:par.nT);
    par.c = opt_vars(par.nT+1:end)';
elseif ~uR_pont && uT_pont
    par.T_uR = opt_vars(1:par.nR);
    par.Ds_uR = opt_vars(par.nR+1:2*par.nR);
    par.T_uT = [];
    par.c = opt_vars(2*par.nR+1:end)';
else
    par.T_uR = opt_vars(1:par.nR);
    par.Ds_uR = opt_vars(par.nR+1:2*par.nR);
    par.T_uT = opt_vars(2*par.nR+1:2*par.nR+par.nT);
    par.c = opt_vars(2*par.nR+par.nT+1:end)';
end

% Clear constants c if maneuver is bang-bang and calling function is STO
if par.isBangbang && strcmp(step,'STO');
    par.c = [];
end
    
% Read maneuver duration T
if ~isempty(T)
    par.T = T;
end

% Abort if switches moved out of maneuver interval (works only if switches
% computed based on switching times)
if ~allowLessSwitches && (...
   any(par.T_uR <= 0) || any(par.T_uR >= par.T) || ...
   any(par.T_uR+par.Ds_uR <= 0) || any(par.T_uR+par.Ds_uR >= par.T) || ...
   any(par.T_uT <= 0) || any(par.T_uT >= par.T))
    res = inf;
    return;
end

% Abort if singular arc duration is below zero
if any(par.Ds_uR < 0)
    res = inf;
    return;
end

% Abort if identical switches exist (program fails otherwise)
if length(par.T_uR) ~= length(unique(par.T_uR)) || ...
length(par.T_uT) ~= length(unique(par.T_uT))
    res = inf;
    return;
end

% Abort if switch of uR within previous singular arc (program fails otherwise)
if ~uR_pont
    [par.T_uR,IX] = sort(par.T_uR);
    par.Ds_uR = par.Ds_uR(IX);
    for i = 2:length(par.T_uR)
        if par.T_uR(i-1) + par.Ds_uR(i-1) >= par.T_uR(i)
            res = inf;
            return;
        end
    end
end

% Integrate quadrotor dynamics
[par.t,par.x] = integrateQuadrotorDynamics(par,par.ODE_options,0,...
                                           par.ODE_residual_meshsize,...
                                           uR_pont,uT_pont);

% Compute state residual
x_diff = par.x(end,1:5)' - par.xT;
res_state = x_diff'*W*x_diff;

% Extract linear conditions if not bang-bang maneuver assumed
if ~par.isBangbang
    [A,r] = exctractParameters(par,uR_pont,uT_pont);
else
    A = []; r = [];
end

% Compute condition residual
if ~isempty(A) && ~isempty(r)
    res_cond = Wc*(A*par.c - r)'*(A*par.c - r);
else
    res_cond = 0;
end

% Return residual
res = res_state + res_cond;

% Plot trajectories
if par.stateFig > -1 || par.inputFig > -1
    global minimumResidual;
    if res < minimumResidual
        minimumResidual = res;
        plotManeuver(par,[par.stateFig,par.inputFig],uR_pont,uT_pont);
        drawnow;
    end
end

% disp(['res = ',num2str(res,5),...
%       ' (res_state = ',num2str(res_state,5),...
%       ', res_cond = ',num2str(res_cond,5),')']);

end




