function uR = rotInput(t,x,par,sing,pont)
%ROTINPUT Compute rotational input uR at time t.
%   uR = rotInput(t,par,sing,pont) computes the rotational input uR at time
%   t, based on the parameters stored in the struct par. The variable
%   sing defines if the singular input is computed or not (only if pont = 0):
%       sing = -1: not singular input
%       sing =  0: define if singular input based on the switching times in par
%       sing = +1: singular input
%   The variable pont defines if the input is computet based on the switching
%   function resulting from Pontryagin's minimum principle or based on the
%   switching times stored in par:
%       pont = 1: calculation based on switching function
%       pont = 0: calculation based on switching times par.T_uR and par.Ds_uR 

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Set pont to default value if empty
if isempty(pont), pont = 0; end

% Compute rotational control input uR based on switching function
if pont
    if x(6) > 0
        uR = par.uR_min;
    else
        uR = par.uR_max;
    end
    return;
end

% Compute rotational control input uR based on the switching times
if sing == 1 || ...
(any((t >= par.T_uR).*(t < (par.T_uR + par.Ds_uR))) && sing == 0)
    if isempty(par.c)
        error('failed on computing uR since par.c missing');
    else
        c = par.c;
    end
    uR = (c(2)*c(3)-c(1)*c(4))/((c(1)^2+c(3)^2)*t^2-...
         2*(c(1)*c(2)+c(3)*c(4))*t+c(2)^2+c(4)^2);
    uR = max(uR,par.uR_min); % check if not outside boundaries (this check
    uR = min(uR,par.uR_max); % violates the minimum principle, but allows a
                             % robust optimization. For the final maneuver,
                             % the singular uR must always be within the
                             % boundaries, otherwise the minimum principle
                             % is not satisfied)
else
    if ~isempty(par.T_uR)
        if mod(par.uR0_ind + sum((t >= par.T_uR).*par.uR_SAswitch),2)
            uR = par.uR_max;
        else
            uR = par.uR_min;
        end
    else
        if par.uR0_ind
            uR = par.uR_max;
        else
            uR = par.uR_min;
        end
    end
end

end

