function par = solveBVP(par,uR_pont,uT_pont)
%SOLVEBVP Solve the BVP for time-optimal quadrotor maneuvers.
%   par = solveBVP(par,uR_pont,uT_pont) solves the minimum principle BVP to
%   obtain a quadrotor maneuver that is a candidate for time-optimality.
%   For the BVP solver to converge, a good initial guess of the switching
%   times and the constats c are required and must be stored the input par.
%   The resulting maneuver trajectories are returned in the output par.

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Initialize global variables with plotting infos
if par.stateFig > -1 || par.inputFig > -1
    global minimumResidual; %#ok<TLEV> % minimum residual of the current minimization
    minimumResidual = inf;
end

% Define function to minimize
if par.isBangbang
    if uR_pont && uT_pont
        opt_vars = ones(1,4+1);
        res = @(opt_vars) residualFunction(...
                            par,opt_vars(1:end-1).*par.c',...
                            opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
    else
        error(['for the BVP solver both inputs must be determined ',...
               'based on c in the bang-bang case']);
    end
else
    if ~par.STO_sym % optimize all switches if maneuver not symmetric
        if uR_pont && uT_pont
            opt_vars = ones(1,4+1);
            res = @(opt_vars) residualFunction(...
                                par,opt_vars(1:end-1).*par.c',...
                                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        elseif uR_pont && ~uT_pont
            opt_vars = ones(1,par.nT+4+1);
            res = @(opt_vars) residualFunction(...
                                par,opt_vars(1:end-1).*[par.T_uT,par.c'],...
                                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        elseif ~uR_pont && uT_pont
            opt_vars = ones(1,2*par.nR+4+1);
            res = @(opt_vars) residualFunction(...
                                par,opt_vars(1:end-1).*[par.T_uR,par.Ds_uR,par.c'],...
                                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        else
            opt_vars = ones(1,2*par.nR+par.nT+4+1);
            res = @(opt_vars) residualFunction(...
                                par,opt_vars(1:end-1).*[par.T_uR,par.Ds_uR,par.T_uT,par.c'],...
                                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        end
    else % optimize only first half of switches if maneuver symmetric
        if uR_pont && uT_pont
            opt_vars = ones(1,4+1);
            res = @(opt_vars) residualFunction(...
                                par,opt_vars(1:end-1).*par.c',...
                                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        elseif uR_pont && ~uT_pont
            opt_vars = ones(1,par.nT/2+4+1);
            res = @(opt_vars) residualFunction(par,[...
                opt_vars(1:par.nT/2).*par.T_uT(1:par.nT/2),...
                par.T-opt_vars(1:par.nT/2).*par.T_uT(1:par.nT/2),...
                opt_vars(par.nT/2+1:end-1).*par.c'],...
                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        elseif ~uR_pont && uT_pont
            opt_vars = ones(1,par.nR+4+1);
            res = @(opt_vars) residualFunction(par,[...
                opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2),...
                par.T-opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2)-...
                    opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR+1:end-1).*par.c'],...
                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        else
            opt_vars = ones(1,par.nR+par.nT/2+4+1);
            res = @(opt_vars) residualFunction(par,[...
                opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2),...
                par.T-opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2)-...
                    opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR+1:par.nR+par.nT/2).*par.T_uT(1:par.nT/2),...
                par.T-opt_vars(par.nR+1:par.nR+par.nT/2).*par.T_uT(1:par.nT/2),...
                opt_vars(par.nR+par.nT/2+1:end-1).*par.c'],...
                opt_vars(end)*par.T,'BVP',uR_pont,uT_pont);
        end
    end
end
    
% Minimize residual (fminsearch used since gradient methods problematic
% due to numerical noise caused by integration of quadrotor dynamics)
[opt_vars,res_min] = fminsearch(res,opt_vars,par.BVP_options);

% Read optimal switching times and constants
if par.isBangbang
    if uR_pont && uT_pont
        par.c = opt_vars(1:end-1)'.*par.c;
        par.T = opt_vars(end)*par.T;
    else
        error(['for the BVP solver both inputs must be determined ',...
               'based on c in the bang-bang case']);
    end
else
    if ~par.STO_sym
        if uR_pont && uT_pont
            par.c = opt_vars(1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        elseif uR_pont && ~uT_pont
            par.T_uT = opt_vars(1:par.nT).*par.T_uT;
            par.c = opt_vars(par.nT+1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        elseif ~uR_pont && uT_pont
            par.T_uR = opt_vars(1:par.nR).*par.T_uR;
            par.Ds_uR = opt_vars(par.nR+1:2*par.nR).*par.Ds_uR;
            par.c = opt_vars(2*par.nR+1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        else
            par.T_uR = opt_vars(1:par.nR).*par.T_uR;
            par.Ds_uR = opt_vars(par.nR+1:2*par.nR).*par.Ds_uR;
            par.T_uT = opt_vars(2*par.nR+1:2*par.nR+par.nT).*par.T_uT;
            par.c = opt_vars(2*par.nR+par.nT+1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        end
    else
        if uR_pont && uT_pont
            par.c = opt_vars(1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        elseif uR_pont && ~uT_pont
            par.T_uT = [...
                opt_vars(1:par.nT/2).*par.T_uT(1:par.nT/2),...
                par.T-opt_vars(1:par.nT/2).*par.T_uT(1:par.nT/2)];
            par.c = opt_vars(par.nT/2+1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        elseif ~uR_pont && uT_pont
            par.T_uR = [...
                opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2),...
                par.T-opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2)-...
                    opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2)];
            par.Ds_uR = [...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2)];
            par.c = opt_vars(par.nR+1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        else
            par.T_uR = [...
                opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2),...
                par.T-opt_vars(1:par.nR/2).*par.T_uR(1:par.nR/2)-...
                    opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2)];
            par.Ds_uR = [...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2),...
                opt_vars(par.nR/2+1:par.nR).*par.Ds_uR(1:par.nR/2)];
            par.T_uT = [...
                opt_vars(par.nR+1:par.nR+par.nT/2).*par.T_uT(1:par.nT/2),...
                par.T-opt_vars(par.nR+1:par.nR+par.nT/2).*par.T_uT(1:par.nT/2)];
            par.c = opt_vars(par.nR+par.nT/2+1:end-1)'.*par.c;
            par.T = opt_vars(end)*par.T;
        end
    end
end
    
% Store the residual
par.BVP_Res = res_min;

% Clear global variables with plotting infos
if par.stateFig > -1 || par.inputFig > -1
    clear global minimumResidual;
end

% Eliminate nonrelevant switches
if ~uR_pont % only adjust if conrol computed based on switching times
    if mod(sum(par.T_uR <= 0),2) == 1 % adjust uR(0) if necessary
        par.uR0_ind = ~par.uR0_ind;
    end
    par.Ds_uR = par.Ds_uR(par.T_uR > 0); par.Ds_uR = par.Ds_uR(par.T_uR < par.T);
    par.T_uR = par.T_uR(par.T_uR > 0); par.T_uR = par.T_uR(par.T_uR < par.T);
else
    IX = (par.T_uR+par.Ds_uR) > par.T;             % make sure that no switching
    par.T_uR(IX) = par.T - par.Ds_uR(IX) - 10*eps; % times above T exist
end
if ~uT_pont % only adjust if conrol computed based on switching times
    if mod(sum(par.T_uT <= 0),2) == 1 % adjust uT(0) if necessary
        par.uT0_ind = ~par.uT0_ind;
    end
    par.T_uT = par.T_uT(par.T_uT > 0); par.T_uT = par.T_uT(par.T_uT < par.T);
else
    IX = par.T_uT > par.T;         % make sure that no switching times
    par.T_uT(IX) = par.T - 10*eps; % above T exist
end

% Store number of switches
par.nR = length(par.T_uR);                % number of switches in uR
par.nRs = sum(par.Ds_uR > par.PE_TolDs);  % number of singular arcs
par.nT = length(par.T_uT);                % number of switches in uT

% Compute resulting quadrotor trajectories
[par.t,par.x] = integrateQuadrotorDynamics(par,par.ODE_options,0,...
                                           par.ODE_plot_meshsize,...
                                           uR_pont,uT_pont);
par.u = zeros(length(par.t),2);
for i = 1:length(par.t)
    par.u(i,1) = rotInput(par.t(i),par.x(i,:),par,0,uR_pont);
    par.u(i,2) = thrustInput(par.t(i),par.x(i,:),par,uT_pont);
end
par.Phi = [par.x(:,6),...
           (par.c(2)-par.c(1)*par.t).*sin(par.x(:,5))+...
           (par.c(4)-par.c(3)*par.t).*cos(par.x(:,5))];
par.x = par.x(:,1:5); % PhiR stored in par.Phi now

end
