function par = transformParameters(par,dim,omega_max,g)
%TRANSFORMPARAMETERS Transform maneuver and computation parameters
%   par = transformParameters(par,dim,omega_max,g) transforms the maneuver
%   and computation parameters stored in the struct par. The input
%   paramteter dim must be used to define if the values are dimensionalized
%   or nondimensionalized:
%       dim = 1 -> dimensionalize
%       dim = 0 -> nondimensionalize

% PERFORMANCE BENCHMARKING OF QUADROTOR SYSTEMS USING TIME-OPTIMAL CONTROL
% Markus Hehn, Robin Ritz, Raffaello D'Andrea
% Institute for Dynamic Systems and Control
% ETH Zurich
% July 2011

% Check if valid input
if dim
    if ~par.hasDim, dim = 1;
    else error('parameters are already dimensionalized'); end
else
    if par.hasDim, dim = -1;
    else error('parameters are already non-dimensionalized'); end
end

% Transform time vector t
par.t = par.t*(1/omega_max)^dim; % [s]

% Transform state trajectories x
par.x(:,[1,3]) = par.x(:,[1,3])*(g/omega_max^2)^dim; % [m]
par.x(:,[2,4]) = par.x(:,[2,4])*(g/omega_max)^dim;   % [m/s]

% Transform initial and final state vectors
par.x0([1,3]) = par.x0([1,3])*(g/omega_max^2)^dim; % [m]
par.x0([2,4]) = par.x0([2,4])*(g/omega_max)^dim;   % [m/s]
par.xT([1,3]) = par.xT([1,3])*(g/omega_max^2)^dim; % [m]
par.xT([2,4]) = par.xT([2,4])*(g/omega_max)^dim;   % [m/s]

% Transform input trajectories u
par.u(:,1) = par.u(:,1)*omega_max^dim; % [rad/s]
par.u(:,2) = par.u(:,2)*g^dim;         % [m/s^2]

% Transform input boundaries
par.uR_min = par.uR_min*omega_max^dim; % [rad/s]
par.uR_max = par.uR_max*omega_max^dim; % [rad/s]
par.uT_min = par.uT_min*g^dim;         % [m/s^2]
par.uT_max = par.uT_max*g^dim;         % [m/s^2]

% Transform constants c
par.c([1,3]) = par.c([1,3])*(omega_max^2/g)^dim; % [1/m]
par.c([2,4]) = par.c([2,4])*(omega_max/g)^dim;   % [s/m]

% Transform cost value
par.cost = par.cost*omega_max^dim; % [1/s]

% Transform time values
par.T = par.T*(1/omega_max)^dim;                           % [s]
par.STO_deltaT = par.STO_deltaT*(1/omega_max)^dim;         % [s]
par.STO_deltaT_ini = par.STO_deltaT_ini*(1/omega_max)^dim; % [s]
par.T_uR = par.T_uR*(1/omega_max)^dim;                     % [s]
par.Ds_uR = par.Ds_uR*(1/omega_max)^dim;                   % [s]
par.T_uT = par.T_uT*(1/omega_max)^dim;                     % [s]

% Flag as dimensionalized or nondimensionalized
if dim, par.hasDim = 1;
else par.hasDim = 0; end

end

